"""empty message

Revision ID: 358d71fc1759
Revises: ac840f4e3061
Create Date: 2024-02-13 00:37:33.658319

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '358d71fc1759'
down_revision = 'ac840f4e3061'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('notifications', sa.Column('authed_user_id', sa.BigInteger(), nullable=True), schema='fansly')
    op.create_foreign_key(None, 'notifications', 'users', ['authed_user_id'], ['id'], source_schema='fansly', referent_schema='fansly')
    op.add_column('notifications', sa.Column('authed_user_id', sa.BigInteger(), nullable=True), schema='onlyfans')
    op.create_foreign_key(None, 'notifications', 'users', ['authed_user_id'], ['id'], source_schema='onlyfans', referent_schema='onlyfans')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, 'notifications', schema='onlyfans', type_='foreignkey')
    op.drop_column('notifications', 'authed_user_id', schema='onlyfans')
    op.drop_constraint(None, 'notifications', schema='fansly', type_='foreignkey')
    op.drop_column('notifications', 'authed_user_id', schema='fansly')
    # ### end Alembic commands ###
